import os
import numpy as np
from numpy.linalg import norm

def splitWords(text):
    res_words = list()

    # Loop through each line of the file 
    for line in text: 
        # Remove the leading spaces and newline character 
        line = line.strip() 
  
        # Convert the characters in line to 
        # lowercase to avoid case mismatch 
        line = line.lower() 
  
        # Split the line into words 
        sentences = line.split(".")

        for sentence in sentences:
            if len(sentence) == 0:
                continue

            words = sentence.split(" ")

            # Iterate over each word in line 
            for word in words:
                if word == ',' or word == '':
                    continue

                res_words.append(word)

    return res_words

def addWordtoDict(text):
    words_dict = dict()

    words = splitWords(text)

    for word in words:
        if word in words_dict:
            words_dict[word] += 1
        else:
            words_dict[word] = 1

    return words_dict

def mergeTwoDicts(dict_a, dict_b):
    d = dict()

    for word in dict_a:
        if word in d:
            d[word] += dict_a[word]
        else:
            d[word] = dict_a[word]

    for word in dict_b:
        if word in d:
            d[word] += dict_b[word]
        else:
            d[word] = dict_b[word]

    return d

def vectorize_count(dict_a, merged_dict):
    res_vector = list()

    for word in merged_dict:
        if word in dict_a:
            res_vector.append(dict_a[word])
        else:
            res_vector.append(0);

    return np.array(res_vector)


def compute_storage(words_dict_a):
    total_bytes = 0
    
    for word in words_dict_a:
        total_bytes += len(word)

    total_bytes += (len(words_dict_a) * 4)

    return total_bytes

def compute_length(words_dict_a):
    word_length = dict()

    for word in words_dict_a:
        word_length[word] = len(word)

    return word_length

if __name__ == "__main__":

    output = open("cosine_similarity_result.txt", "w")

    for i in range(1, 11):
        for j in range(3, 9):
              file_a = "./data/paper_"+str(i)+".txt"
              file_b = "./data/paper_"+str(i)+"_D"+str(j)+".txt"

              # Open the file in read mode 
              text_a = open(file_a, "r") 
              text_b = open(file_b, "r")

              # Construct word dictionary
              words_dict_a = addWordtoDict(text_a)
              words_dict_b = addWordtoDict(text_b)

              print("Length of word dictionary a: ", len(words_dict_a))
              print(words_dict_a)
              print("Length of word dictionary b: ", len(words_dict_b))
              print(words_dict_b)

              total_bytes = compute_storage(words_dict_a)
              print("Total bytes of words_dict_a: ")
              print(total_bytes)

              word_length = compute_length(words_dict_a)
              print("Word length of words_dict_a: ")
              print(word_length)

              merged_dict = mergeTwoDicts(words_dict_a, words_dict_b)

              print("merged dictionary: ", len(merged_dict))
              print(merged_dict)

              # Construct vector of word frequency
              word_frequency_A = vectorize_count(words_dict_a, merged_dict)
              word_frequency_B = vectorize_count(words_dict_b, merged_dict)

              print("Vector for text_a:")
              print(word_frequency_A)
              print("Vector for text_b:")
              print(word_frequency_B)

              # compute cosine similarity
              cosine_word_frequency = np.dot(word_frequency_A, word_frequency_B) / \
                                      (norm(word_frequency_A) * norm(word_frequency_B))

              out_str = "Cosine Similarity: " + file_a + " " + file_b + " " + str(cosine_word_frequency) + "\n"
              output.write(out_str)